package configurations;

import consumer.CsvFileReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.kafka.annotation.EnableKafka;
import org.springframework.kafka.core.DefaultKafkaProducerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.ProducerFactory;
import util.KafkaProperties;
import util.PropertyToMapConverter;

import java.util.Map;

@PropertySource("classpath:application.properties")
@Configuration
@EnableKafka
public class KafkaProducerConfig {

    @Autowired
    private Environment env;

    @Bean
    public KafkaProperties kafkaProducerProp()  {
        return new KafkaProperties(env.getProperty("ProducerFile"));
    }

    @Bean
    public Map<String, Object> producerConfigs() {
        return new PropertyToMapConverter(kafkaProducerProp().getProperties()).getKafkaProducerProperties();
    }

    @Bean
    public ProducerFactory<String, String> producerFactory() {
        return new DefaultKafkaProducerFactory<>(producerConfigs());
    }

    @Bean
    public KafkaTemplate<String, String> kafkaTemplate() {
        return new KafkaTemplate<>(producerFactory());
    }

    @Bean
    public String outputTopicForProcessing()
    {
        return env.getProperty("outputTopic");
    }

    @Bean
    public CsvFileReader csvFileReader() {
    return new CsvFileReader();
    }
}

