import kafka.serializer.StringDecoder;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.spark.SparkConf;
import org.apache.spark.streaming.Durations;
import org.apache.spark.streaming.api.java.JavaPairInputDStream;
import org.apache.spark.streaming.api.java.JavaStreamingContext;
import org.apache.spark.streaming.kafka.KafkaUtils;
import scala.Tuple2;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

public class SparkKakfaIngestionMain {

    public static void main(String[] args) {
        SparkConf sparkConf = new SparkConf().setMaster("local[4]")
                .setAppName("SparkKafkaIngestion")
                .set("spark.streaming.stopGracefullyOnShutdown","true");


        JavaStreamingContext jssc = new JavaStreamingContext(sparkConf, Durations.seconds(2));
        //TODO
        //jssc.checkpoint("D:\\projects\\aws-capacitor\\basicsparkingestion\\checkpoint");

        Set<String> topicsSet = new HashSet<>();
        topicsSet.add("inputForProcessing");
        Map<String, String> kafkaParams = new HashMap<>();
        kafkaParams.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, "localhost:9092");
        kafkaParams.put(ConsumerConfig.GROUP_ID_CONFIG, "kafkaIngest");
        kafkaParams.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG,"smallest");

        File directory = new File("D:\\projects\\aws-capacitor\\basicsparkingestion\\dataset");
        File[] lFiles = directory.listFiles();
        HashSet<String> folders = new HashSet<>();

        for (File lFile : lFiles) {
            if(lFile.isDirectory())
            {
                folders.add(lFile.getName());
            }
        }

        SimpleDateFormat st = new SimpleDateFormat("dd-MM-yyyy_H");
        int offset = 5*1000*60*60+30*1000*60;

        // Create direct kafka stream with brokers and topics
        JavaPairInputDStream<String, String> messages = KafkaUtils.createDirectStream(
                jssc,
                String.class,
                String.class,
                StringDecoder.class,
                StringDecoder.class,
                kafkaParams,
                topicsSet
        );

        messages.foreachRDD( x ->{
            x.mapToPair(y-> {
                Date dt = new Date(Long.parseLong(y._1)- offset);
                String keyName = st.format(dt);
                return new Tuple2<String,StringBuilder>(keyName,new StringBuilder(y._2));
            })
                    .reduceByKey((a,b)->a.append(System.lineSeparator()+b))
                    .foreach((a)->{
                        if(!folders.contains(a._1))
                        {
                            //Todo
                            File newDirectory = new File("D:\\projects\\aws-capacitor\\basicsparkingestion\\dataset\\"+a._1);
                            if(newDirectory.mkdir())
                                folders.add(a._1);
                            else
                            {
                                //TODO logging
                                System.out.println("couldn't create folder for "+ a._1);
                            }

                        }
                        //TODO
                        try(BufferedWriter writer = new BufferedWriter(new FileWriter
                                ("D:\\projects\\aws-capacitor\\basicsparkingestion\\dataset\\"+a._1+"\\part_"+System.currentTimeMillis())))
                        {
                            writer.write(a._2.toString());
                        } catch (IOException e) {
                            //TODO logging
                            e.printStackTrace();
                        }
                    });
        });

        jssc.start();
        try {
            jssc.awaitTermination();
        } catch (InterruptedException e) {
            System.out.println("Interrupted");
        }

    }
}
